#ifndef Automaton_H
#define Automaton_H

#include "Map.h"
#include "Topology.h"
#include "Neighborhood.h"
#include "TowerNeighborhood.h"

#include <vector>

///The Automaton class is the main class in the library, represents a Cellular Automata type. 
/**
 * This class uses a type of neighborhood and a type of topology in a map 
 * class, to create the final graph, this graph is a net of references to 
 * a matrix, the base element matrix. 
 * This class contain the next step function, the method to calculate the 
 * next inner states of the elements in the base element matrix, given by 
 * the graph form and the change state algorithm given by the method. 
*/ 
template<class T> class Automaton{
	
public:
    //--------------------start of the inner variables--------------------------
	
	///This vector is used in createNode() method for a lack of compatibility with the Map class, soon is going to be eliminated. 
	std::vector<int> position;    
	///This structure represent a vertex in graph with all is properties.
	/**
	* The Node objects represents the node in the graph, 
	* contain a pointer to a state value in the values matrix,
	* and a pointers to other states in the matrix representing 
	* the edge structure
	*/
    typedef struct  Node{
      
      public:
          
      ///The pointer to the game board
      T* centerCell;
      ///The end index in the vector of Neighbor's pointer
      int endNeighbor;
      ///The begin index in the vector of Neighbor's pointer
      int beginNeighbor;
	  
	  ///The constructor.
	 /** The constructor input, are a pointer to the states matrix, 
	 * the vertex in the graph, and two ints variables representing 
	 * the start and end index of the Neighbors pointer container, 
	 * this is the representation of the edges for this graph element, 
	 * the pointer "package" that is the Neighborhood.    
	 */
	  Node(T* cell, int begin, int end):centerCell(cell), beginNeighbor(begin), endNeighbor(end){}
      
    };
    /********************************************************/

	///The pointer to the states matrix.
	/**
	* This pointer is used to move across the states matrix, in this 
	* way the program works independently of the dimension of the 
	* states matrix. 
	*/
    T *pointerToNodes;
	///This is the pointer that is actually moved across the state matrix. 
	/**
	* Is used to keep the reference to the start of the matrix given
	* by pointerToNodes variable.
	*/
	T *copyPointerToNodes;
	///In createNode()there is a need to an extra copy of copyPointerToNodes variable.
	T *copyPointerToNodes2;
    ///The pointers container to the state matrix. 
	/** Represents the edges in the graph, the neighbors in the cellular automata */
	std::vector<T*> neighbors;
    ///the Node container.
	/**
	* The Node structure is equal to the node of a graph, has inner state value 
	* and edges to other elements, and the graph vector is the container of Node 
	* structures.
	*/ 
    std::vector<Node> graph;    
	/// The maximum dimensions number.
	/**
	* To accomplish all the calculations and maintain the possibility to use 
	* n-dimensional matrixes, we create an array to keep all the dimension sizes, 
	* this array need an initial size value a const int variable type; 
	* The maxDimensions is this type, so for now we have the possibility to use a 
	* maximum dimensions number of 100.  
	*/
    static const int maxDimensions = 100;
    /// The array of dimensions sizes.
	/**
	* Every element in the dimSizes array represents a size of some dimension; 
	* This array has fixed long size, the maxDimensions variable, if the dimension 
	* variable, the dimension for this Automaton object is smaller than the size 
	* of this array, each element pass the dimension number is not taken account 
	* and is ignored. The Automaton dimension cant be bigger than the size of this 
	* array.
	*/   
	int dimSizes[maxDimensions];
    ///A copy of dimSizes array is used to apply some changes in the dimSizes array without lost is information.
	int copyDimSizes[maxDimensions];
	///The number of dimensions in the Automaton.
	int dimension;
	///Index value to determine starts and finish int values in the Node structure.
	/**
	* This int value is to accomplish the calculations and keep track of where starts,
	* and where finish a neighborhood in the vertex's vector container 
	* (in the createNode() method).
	*/
    int indexHood;    
    ///The Automaton needs a Map object to determine the actual form of the graph. 
	/** 
	* With this object the form of the graph is developed in base of the 
	* "desired" form given by the Neighborhood object and the "possible" 
	* form given by the topology object. 
	*/
	Map *map;
	///The Topology object determines the possible form of the graph.
    Topology *topology;
    ///The Neighborhood object determines the desired form of the graph. 
	Neighborhood *neighborhood;     
    ///The states variable is for combinatory
    int states;
    //--------------end of the inner variables-----------------------------
                
public:
	///The constructor 
    Automaton(T *gridPointer, int Dsizes[], int dimesions):pointerToNodes(gridPointer), dimension(dimesions){
    /*T *gridPointer() is the moving pointer over the
	grid, used like an iterator, in "ndimensional arrays",
	Dsizes[] is the pointer to the dimension sizes array,
	and int dimension is the number of dimensions of the 
	array */
			
		//set the index of neighbors to 0
		indexHood = 0;

		//create the dimension objects
		//one for each domension
		for(int i=0;i < dimension; ++i){
			dimSizes[i] = Dsizes[i];              
		}
		//then create a toppolgy objects and put in 
		//the dimension objects
        topology = new Topology(dimension);
        for(int i=0;i<dimension;++i){
			topology->setDimensionSize(i, Dsizes[i]);
		}
		//last create a neighborhood object and create
		//the map with the topology and neighborhood objects
        neighborhood = new TowerNeighborhood(); 
        map = new Map(topology, neighborhood);

		//Reserve space to avoid copy a lot memory if the size of the graph result to big.
		int graphSize = 1;
		for(int i=0;i < dimension; ++i){
			graphSize = graphSize * dimSizes[i];
		}
		graph.reserve(graphSize);
	}
    
    /// The copy-constructor:
    Automaton(const Automaton& a){
		pointerToNodes = a.pointerToNodes; 
		dimension = a.dimension;
		topology = a.topology;
		dimSizes = a.dimSizes;
		neighborhood = a.neighborhood;
		map = a.map;
		graph.reserve(a.graph.size());
		//set the index of neighbors to 0
		indexHood = 0;

	}
    
	///Overwrite the = operator to avoid problems when you forget initialize whit the constructor  
    Automaton& operator=(const Automaton& a) {
		pointerToNodes = a.pointerToNodes; 
		dimension = a.dimension;
		topology = a.topology;
		dimSizes = a.dimSizes;
		neighborhood = a.neighborhood;
		map = a.map;
		graph.reserve(a.graph.size());
		//set the index of neighbors to 0
		indexHood = 0;
        return *this;
    }   
	///Set the Map objects, that unifies the desired form of the graph given by the Topology with the possible form given by the Neighborhood.
	void setMap();
    ///Return the Map "mixer" object of this Automaton.
	void getMap();
    ///Set the Neighborhood object of this class, that determine the desired form of the graph.
	void setNeighborhood();
    ///Return the Neighborhood object of this Automaton.
	void getNeighborhood();
    ///With this function you can iterate over an array independently of the dimension and element type of the array.
	/**This is done tanks to the pointer to the first element of the 
	* array pointerToNodes, and the dimension sizes array dimSizes[maxDimensions].
	* The pointer as starting point reference and the array is for know 
	* how many places move the pointer to go from one position to 
	* other in the states matrix, because the multidimensional arrays 
	* are keep has memory places one next to the other.
    * In this method you iterate over the states array calling the 
	* createNode() method, for each element in the array, each element 
	* is a "center cell", the "nodes" in the graph.
	* All the edges and final graph structure is set in every time is call the createNode() method. 
	**/
	void iterateOverTheSatesArray(int index=0);
	//ToDo:
	void nextStep();
	///This method is for creating the structure (the Node) container of the couple's node and edges.
	/** 
	* The "structures" container is the graph, and the structure 
	* consist of a center pointer to the states matrix, the node,
	* and two index for the beginning and end of the neighbors 
	* positions index in the vector container, the edges (links 
	* to neighbors).
    *
	* This method use the copyDimSizes position array, this vector 
	* define the node coordinates in the grid, and the Map class 
	* is used to find if the edge exist and in the case generate 
	* is coordinates, Map is the uses the Topology class and the 
	* Neighborhood class.
	* The generated edge (pointer to element in the state array) 
	* is keep, in the neighbors vector container, and the index 
	* values in the Node structure in particular the end index is modify.
	*/
	void createNode();
	/// This method change the copyPointerToNodes pointer to point a element in the states array given by the position argument.
	/**
	* If you want to access any element in a multi-dimensional array, 
	* use a pointer to the first element and advance the pointer to
	* the position of the element that you want to change/access. 
	* The reference pointer is pointerToNodes, mark the start bunch 
	* of memory use by the states array, and the values to know how 
	* much positions move to pass from one dimension to the next one 
	* in the array is give by dimSizes array.
	* The pointer copyPointerToNodes finish pointing to the desired 
	* position in the states array.
	*/
    void getNodePointer(std::vector<int> position);	
	///This method is equal to getNodePointer except use an array as n input instead a vector
	/**
	* Watch the getNodePointer method documentation.
	*/
    void getNodePointer(int position[]);
	///This method do the same that getNodePointer(int position[]) but use copyDimSizes array instead input.
	/**
	* Use the copyDimSizes array as the coordinates of 
	* the element in the state array to point to in 
	* copyPointerToNodes pointer.
	*/
	void getNodePointer();
    
};
template<class T>
void Automaton<T>::getNodePointer()
    {
    
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes; 
        /*whe need a int to count the positions 
		to move the pointer */
        int move = 0 ;
        
        //for all the elements in the given position
		//(the copyDimSizes position variable)
		//the copyDimSizes size is = dimension 
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension){
                //multiply all the position index's for 
                //each dimension size    
                move = copyDimSizes[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                  copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += copyDimSizes[i];
        } 
}//---------------getNodePointer()--------------------------

template<class T>   
void Automaton<T>::getNodePointer(std::vector<int> position)
    {
    
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes;
        /*whe need a int to count the positions 
		to move the pointer */
        int move = 0 ;
        
        //for all the elements in the given position
		//(vector<int> position argument)
		//the position size is = dimension 
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension ){
                //multiply all the position index's for 
                //each dimension size    
                move = position[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                //and move that number of times the pointer                                    
                copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += position[i];
        } 
    }//---------------getNodePointer(vector<int> position)--------------------------

template<class T>   
void Automaton<T>::getNodePointer(int position[])
    {
       
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes;
        //whe need a int to count the positions to move the
        //pointer 
        int move = 0 ;
        
        //for all the elements in the given positions
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension ){
                //multiply all index of the position for the 
                //dimension of is respective dimension in the array     
                move = position[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                //and move that number of times the pointer                                    
                copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += position[i];
        } 
    }//---------------getNodePointer(int position[])--------------------------   
template<class T>
void Automaton<T>::createNode(){
    
	//first get the pointer to the node "center cell"
	//copyPointerToNodes pointer
	getNodePointer();
	//because whe are goin to change copyPointerToNodes pointer
	//first keep the pointing direction
	copyPointerToNodes2 = copyPointerToNodes;
	
	
	//yo have to transform the position vector, the arraycopyDimSizes[]
	//to a vector container for use for the map
    position.clear();
    position.reserve(dimension);
    for(int i=0;i<dimension;++i){
        position.push_back(copyDimSizes[i]);
    }
	//calculate the neighborhood
    map->setEdge(position);
    std::vector< std::vector<int> > hood = map->getEdge();
	
	//next keep the edge (the pointer) in the neighbors container

	//for all the neighbors directions
    for(int i=0;i<int(hood.size());++i){
		//move the pointer to the element given by the direction
        getNodePointer(hood[i]);
		//and push the pointer in the container
        neighbors.push_back(copyPointerToNodes);    
    }
	//create the node
    Node cell(copyPointerToNodes2, indexHood, (indexHood + int(hood.size())-1));
    //and laste change the index array for the next time
	indexHood = indexHood + int(hood.size());
    
	//keep the structure in the grahp vector container
    graph.push_back(cell);
    
}//------------createNode()-----------------

template<class T>
void Automaton<T>::iterateOverTheSatesArray(int index){    
    
         //iterate over the first dimension 
        for( int i=0; i<dimSizes[index] ; ++i ){
            
			//now modify a copy of the dimension sizes array,
			//to use has the new position vector
			copyDimSizes[index] = i;                
                
                //if isn't the last dimension 
                if( (index + 1) < dimension){
                    /*use recursion to modify the copy of 
					the dimension sizes array, and start again
					but for the nexts dimensions
					*/
                    iterateOverTheSatesArray(index+1);                               
                }
                //if is the last dimension
                else{					
			/*modify a copy of the dimension sizes array,
			to use has the new position vector
            in the last dimension*/
					copyDimSizes[index] = i;
                    //and use the index (the new position vector)
                    //to create the Node and edge structure
                    createNode();
                }
        }       
}//----------iterateOverTheSatesArray-----------------

template<class T>
void Automaton<T>::nextStep(){
    
}//-----------nextStep()--------------------

#endif
