#ifndef Neighborhood_h
#define Neighborhood_h

#include <vector>
///This class represents the desired form for the neighborhood.
/**
* This class represents the "form" of the neighborhood 
* for some center cell, the specific edges of a vertex. 
* This class specify the desired form for the neighborhood, then the 
* topology class specify if this form can be created, and 
* how. This mix and the derived correlation's are performed 
* in the map class. 
*/
    
class Neighborhood{
	public:
    
    /**This is the container of neighbor's positions vectors
	the edges positions set, for some center cell, some vertex.*/ 
    std::vector< std::vector<int> > neighborsPositions; 
    
    /**This vector will be used for create a neighbor position
    and push in to the container (neighborsPositions). */
    std::vector<int> neighborCoordinates; 
    
    /**
	* The size variable represents how far we are gonna take 
	* neighbors. For example if we take size = 1, in the tower 
	* neighborhood (1 dimension) you have:                     
	* 0X0 and if we have size = 2 we have: 00X00
    * The x is the center cell, and the 0's are the neighbors 
	*/
    int size;
     
    ///The constructor
	Neighborhood(int size = 1): size(size){} 

	/// The copy-constructor:
    Neighborhood(const Neighborhood& n): size(n.size){}
	
	///Overwrite the = operator to avoid problems when you forget initialize whit the constructor
    Neighborhood& operator=(const Neighborhood& n) {
        size = n.size;
        return *this;
    }   
    
    ///the destructor 
    ~Neighborhood(){
			neighborsPositions.clear(); 
			neighborCoordinates.clear(); 
	}
    
    //------get set methods-----------------------------
	/// Return the container of positions of the neighbors (neighborsPositions).
    inline std::vector< std::vector<int> > getNeighborhood(){return neighborsPositions;}
    ///What size is the diameter of the Neighborhood?
    inline int getSize(){return size;}    
	///Change the size of the diameter of the Neighborhood
	inline void setSize(int s){size = s;}
        
    //------end get set  methods-------------------------         
    
    /**
    * This is the main method in the class.
	* In this method, the container of neighbor positions, 
	* is fill with the vector positions of the neighbors, 
	* the positions of the vertex with an edge to the vertex 
	* represented by the input vector  (the vertex position).   
	*/
    virtual void calculateNeighborhood(const std::vector<int> position) {};          

};

#endif
